
local function push(s, v)
	s[s.n+1], s.n = v, s.n+1
end
local function pop(s)
	local v = s[s.n]
	s[s.n], s.n = nil, s.n-1
	return v
end
local function peek(s)
	return s[s.n]
end

local function nextbyte(s, i)
	i = i + 1
	local b = s:byte(i)
	if b then
		return i, b
	else
		return
	end
end
function string.gbyte(s)
	return nextbyte, s, 0
end
local function B(s) return s:byte() end

local ops = {
	[B'+'] = function(s) push(s,   pop(s)+pop(s)) end,
	[B'-'] = function(s) push(s,  -pop(s)+pop(s)) end,
	[B'*'] = function(s) push(s,   pop(s)*pop(s)) end,
	[B'/'] = function(s) push(s, 1/pop(s)*pop(s)) end,
}

local function evaluate(expr)
	local p = 0
	local yardp = {n=0}
	local yard  = {n=0}
	local stack = {n=0}
	for i, c in expr:gbyte() do
		if     c == B' ' then
			p = p + 1
		elseif c == B'+' or c == B'-' or c == B'*' or c == B'/' then
			local tp = peek(yardp)
			while tp and tp <= p do
				if stack.n > 1 then
					pop(yardp)
					ops[pop(yard)](stack)
				else
					return nan, "unbalanced"
				end
				tp = peek(yardp)
			end
			push(yard, c)
			push(yardp, p)
			p = 0
		elseif B'0' <= c and c <= B'9' then
			push(stack, c-B'0')
			p = 0
		else
			-- ignore unknown char
		end
	end
	while yard.n > 0 and stack.n > 1 do
		ops[pop(yard)](stack)
	end
	if yard.n == 0 and stack.n == 1 then
		return pop(stack)
	else
		return nil, "unbalanced"
	end
end



-- tests --

local tests = {
	"9 / 1+2  -  1    /    2",
	"0-6 * 2",
	"4  /  1 + 1",
	"2-4/2",
	"-4",
	"34",
	"1 / 0",
	"9*9*9*9*9*9*9*9*9*9",
}

for i,test in ipairs(tests) do
	print(i, evaluate(test), "=", test)
end
print("done!")
